@@ -19,7 +19,6 @@ from __future__ import annotations
 
 import logging
 import re
-import struct
 import sys
 from collections.abc import Iterable
 from collections.abc import Sequence
@@ -27,7 +26,7 @@ from typing import Any
 from typing import cast
 
 from mitmproxy import ctx
-from mitmproxy import dns
+from mitmproxy.connection import Address
 from mitmproxy.net.tls import starts_like_dtls_record
 from mitmproxy.net.tls import starts_like_tls_record
 from mitmproxy.proxy import layer
@@ -47,7 +46,7 @@ from mitmproxy.proxy.layers import ServerTLSLayer
 from mitmproxy.proxy.layers import TCPLayer
 from mitmproxy.proxy.layers import UDPLayer
 from mitmproxy.proxy.layers.http import HTTPMode
-from mitmproxy.proxy.layers.quic import quic_parse_client_hello
+from mitmproxy.proxy.layers.quic import quic_parse_client_hello_from_datagrams
 from mitmproxy.proxy.layers.tls import dtls_parse_client_hello
 from mitmproxy.proxy.layers.tls import HTTP1_ALPNS
 from mitmproxy.proxy.layers.tls import HTTP_ALPNS
@@ -128,9 +127,9 @@ class NextLayer:
         # 1)  check for --ignore/--allow
         if self._ignore_connection(context, data_client, data_server):
             return (
-                layers.TCPLayer(context, ignore=True)
+                layers.TCPLayer(context, ignore=not ctx.options.show_ignored_hosts)
                 if tcp_based
-                else layers.UDPLayer(context, ignore=True)
+                else layers.UDPLayer(context, ignore=not ctx.options.show_ignored_hosts)
             )
 
         # 2)  Handle proxy modes with well-defined next protocol
@@ -154,7 +153,7 @@ class NextLayer:
             server_tls.child_layer = ClientTLSLayer(context)
             return server_tls
         # 3b) QUIC
-        if udp_based and _starts_like_quic(data_client):
+        if udp_based and _starts_like_quic(data_client, context.server.address):
             server_quic = ServerQuicLayer(context)
             server_quic.child_layer = ClientQuicLayer(context)
             return server_quic
@@ -166,26 +165,30 @@ class NextLayer:
             return layers.UDPLayer(context)
 
         # 5)  Handle application protocol
-        # 5a) Is it DNS?
-        if udp_based:
-            try:
-                # TODO: DNS over TCP
-                dns.Message.unpack(data_client)  # TODO: perf
-            except struct.error:
-                pass
-            else:
-                return layers.DNSLayer(context)
-        # 5b) We have no other specialized layers for UDP, so we fall back to raw forwarding.
+        # 5a) Do we have a known ALPN negotiation?
+        if context.client.alpn:
+            if context.client.alpn in HTTP_ALPNS:
+                return layers.HttpLayer(context, HTTPMode.transparent)
+            elif context.client.tls_version == "QUICv1":
+                # TODO: Once we support more QUIC-based protocols, relax force_raw here.
+                return layers.RawQuicLayer(context, force_raw=True)
+        # 5b) Is it DNS?
+        if context.server.address and context.server.address[1] in (53, 5353):
+            return layers.DNSLayer(context)
+        # 5c) We have no other specialized layers for UDP, so we fall back to raw forwarding.
         if udp_based:
             return layers.UDPLayer(context)
-        # 5b) Check for raw tcp mode.
-        very_likely_http = context.client.alpn in HTTP_ALPNS
-        probably_no_http = not very_likely_http and (
+        # 5d) Check for raw tcp mode.
+        probably_no_http = (
             # the first three bytes should be the HTTP verb, so A-Za-z is expected.
             len(data_client) < 3
+            # HTTP would require whitespace before the first newline
+            # if we have neither whitespace nor a newline, it's also unlikely to be HTTP.
+            or (data_client.find(b" ") >= data_client.find(b"\n"))
             or not data_client[:3].isalpha()
             # a server greeting would be uncharacteristic.
             or data_server
+            or data_client.startswith(b"SSH")
         )
         if ctx.options.rawtcp and probably_no_http:
             return layers.TCPLayer(context)
@@ -232,6 +235,9 @@ class NextLayer:
                 client_hello := self._get_client_hello(context, data_client)
             ) and client_hello.sni:
                 hostnames.append(f"{client_hello.sni}:{port}")
+            if context.client.sni:
+                # Hostname may be allowed, TLS is already established, and we have another next layer decision.
+                hostnames.append(f"{context.client.sni}:{port}")
 
         if not hostnames:
             return False
@@ -315,7 +321,7 @@ class NextLayer:
                 return None
             case "udp":
                 try:
-                    return quic_parse_client_hello(data_client)
+                    return quic_parse_client_hello_from_datagrams([data_client])
                 except ValueError:
                     pass
 
@@ -342,10 +348,15 @@ class NextLayer:
                     stack /= ClientTLSLayer(context)
                 stack /= HttpLayer(context, HTTPMode.transparent)
             case "https":
-                stack /= ServerTLSLayer(context)
-                if starts_like_tls_record(data_client):
-                    stack /= ClientTLSLayer(context)
-                stack /= HttpLayer(context, HTTPMode.transparent)
+                if context.client.transport_protocol == "udp":
+                    stack /= ServerQuicLayer(context)
+                    stack /= ClientQuicLayer(context)
+                    stack /= HttpLayer(context, HTTPMode.transparent)
+                else:
+                    stack /= ServerTLSLayer(context)
+                    if starts_like_tls_record(data_client):
+                        stack /= ClientTLSLayer(context)
+                    stack /= HttpLayer(context, HTTPMode.transparent)
 
             case "tcp":
                 if starts_like_tls_record(data_client):
@@ -385,7 +396,7 @@ class NextLayer:
             case "quic":
                 stack /= ServerQuicLayer(context)
                 stack /= ClientQuicLayer(context)
-                stack /= RawQuicLayer(context)
+                stack /= RawQuicLayer(context, force_raw=True)
 
             case _:  # pragma: no cover
                 assert_never(spec.scheme)
@@ -417,12 +428,47 @@ class NextLayer:
         )
 
 
-def _starts_like_quic(data_client: bytes) -> bool:
-    # FIXME: handle clienthellos distributed over multiple packets?
-    # FIXME: perf
-    try:
-        quic_parse_client_hello(data_client)
-    except ValueError:
+# https://www.iana.org/assignments/quic/quic.xhtml
+KNOWN_QUIC_VERSIONS = {
+    0x00000001,  # QUIC v1
+    0x51303433,  # Google QUIC Q043
+    0x51303436,  # Google QUIC Q046
+    0x51303530,  # Google QUIC Q050
+    0x6B3343CF,  # QUIC v2
+    0x709A50C4,  # QUIC v2 draft codepoint
+}
+
+TYPICAL_QUIC_PORTS = {80, 443, 8443}
+
+
+def _starts_like_quic(data_client: bytes, server_address: Address | None) -> bool:
+    """
+    Make an educated guess on whether this could be QUIC.
+    This turns out to be quite hard in practice as 1-RTT packets are hardly distinguishable from noise.
+
+    Returns:
+        True, if the passed bytes could be the start of a QUIC packet.
+        False, otherwise.
+    """
+    # Minimum size: 1 flag byte + 1+ packet number bytes + 16+ bytes encrypted payload
+    if len(data_client) < 18:
         return False
+    if starts_like_dtls_record(data_client):
+        return False
+    # TODO: Add more checks here to detect true negatives.
+
+    # Long Header Packets
+    if data_client[0] & 0x80:
+        version = int.from_bytes(data_client[1:5], "big")
+        if version in KNOWN_QUIC_VERSIONS:
+            return True
+        # https://www.rfc-editor.org/rfc/rfc9000.html#name-versions
+        # Versions that follow the pattern 0x?a?a?a?a are reserved for use in forcing version negotiation
+        if version & 0x0F0F0F0F == 0x0A0A0A0A:
+            return True
     else:
-        return True
+        # ¯\_(ツ)_/¯
+        # We can't even rely on the QUIC bit, see https://datatracker.ietf.org/doc/rfc9287/.
+        pass
+
+    return bool(server_address and server_address[1] in TYPICAL_QUIC_PORTS)
