@@ -95,8 +95,8 @@ if urwid.util.detected_encoding:
     SYMBOL_REPLAY = "\u21ba"
     SYMBOL_RETURN = "\u2190"
     SYMBOL_MARK = "\u25cf"
-    SYMBOL_UP = "\u21E7"
-    SYMBOL_DOWN = "\u21E9"
+    SYMBOL_UP = "\u21e7"
+    SYMBOL_DOWN = "\u21e9"
     SYMBOL_ELLIPSIS = "\u2026"
     SYMBOL_FROM_CLIENT = "\u21d2"
     SYMBOL_TO_CLIENT = "\u21d0"
@@ -188,7 +188,7 @@ class TruncatedText(urwid.Widget):
             text = text[::-1]
             attr = attr[::-1]
 
-        text_len = urwid.util.calc_width(text, 0, len(text))
+        text_len = urwid.calc_width(text, 0, len(text))
         if size is not None and len(size) > 0:
             width = size[0]
         else:
@@ -762,7 +762,7 @@ def format_flow(
             duration = f.messages[-1].timestamp - f.client_conn.timestamp_start
         else:
             duration = None
-        if f.client_conn.tls_version == "QUIC":
+        if f.client_conn.tls_version == "QUICv1":
             protocol = "quic"
         else:
             protocol = f.type
